@@ -4,14 +4,7 @@ import torch.nn.functional as F
 
 import triton
 import triton.language as tl
-
-def calc_num_warps(block_size):
-    num_warps = 4
-    if block_size >= 2048:
-        num_warps = 8
-    if block_size >= 4096:
-        num_warps = 16
-    return num_warps
+from triton_transformer.utils import calc_num_warps
 
 @triton.jit
 def softmax_kernel_forward(
